#!/usr/bin/env python
"""

    A collection of utility functions useful for file name and path
    name manipulation.
    
    :Author: Christopher Hanley
    
"""
from __future__ import division
import os

import pyfits
import error

# Helper Functions

instlettercodes = {
    "acs":"j",
    "foc":"x",
    "fos":"y",
    "hrs":"z",
    "hsp":"v",
    "nic":"n",
    "stis":"o",
    "synphot":"m",
    "wfpc2":"u",
    "wfpc":"w",
    "wfc3":"i",
    "cos":"l"
    }

letterinstcodes = {
    "j":"acs",
    "x":"foc",
    "y":"fos",
    "z":"hrs",
    "v":"hsp",
    "n":"nic",
    "o":"stis",
    "m":"synphot",
    "u":"wfpc2",
    "w":"wfpc",
    "i":"wfc3",
    "l":"cos"
    }

def updatelog(log,message):
    """
    
    Purpose
    =======
    Given a message an a log file name write the message to the log file.
    If no log file name is given then print the message to stdout.
    
    """
    if log:
        file = open(log,'a')
        file.write(message)
        file.close()
    else:
        print message


def fileinfo(pathfilename):
    """
    
    Purpose
    =======
    Given a file name, absolute or relative, return the absolute path name,
    path name, and file name of the input file name.
    
    """
    fullpathfilename = os.path.realpath(pathfilename)
    filename = os.path.basename(pathfilename)
    path = os.path.dirname(os.path.realpath(pathfilename))
    return (fullpathfilename, path, filename)

def fixedfilename(filename, ident):
    """
    
    Purpose
    =======
    Given a FITS file name add a ".fixed" extension ahead of the
    ".fits" extension.
    
    """
    fnamelist = filename.split('.')
    fnamelist.insert(-1, ident)
    return ".".join(fnamelist)

def getlodfilename(pathfilename):
    """
    
    Purpose
    =======
    Given the name of a FITS file return the name of the lod file
    that will be created by CDBS processing.
    
    """
    (fullpathfilename, path, filename) = fileinfo(pathfilename)
    fnamelist = filename.split('.')
    fnamelist[-1] = 'lod'
    newroot = ".".join(fnamelist)
    return path+'/'+newroot

def getfiletype(filename):
    """
    
    Purpose
    =======
    Determine if the given reference file is a table file or image
    file.
    
    """
    hdulist = pyfits.open(filename)
    
    if isinstance(hdulist[1], pyfits.NP_pyfits.ImageHDU):
        return "image"
    elif isinstance(hdulist[1], pyfits.NP_pyfits.BinTableHDU):
        return "table"
    else:
        err = "Error in getfiletype. Not supported file."
        out = err
        raise error.CDBSError(out, err, error.PYERRORCODE)


def getinstname(filename):
    """
    
    Purpose
    =======
    Determine the type of HST instrument the reference file is
    created to support.
    
    """
    hdulist = pyfits.open(filename)
    return hdulist[0].header['instrume'].strip().lower()

    
def asciifilesmatch(file1,file2):
    """
    
    Purpose
    =======
    Do a line by line comparison of two acsii files to determine if
    they are identical.
    
    """
    
    file1 = open(file1)
    file2 = open(file2)
    
    f1 = file1.readlines()
    f2 = file2.readlines()
    
    if len(f1) != len(f2):
        raise ValueError, "Files are of different length."
    
    for line in range(len(f1)):
        if f1[line] != f2[line]:
            raise ValueError, "Files differ at line: "+str(int(line)+1)
    return True

def getcurdir():
    """
    
    Purpose
    =======
    Determine the name of the current directory and return it as a string.
    
    """
    return os.path.abspath(os.path.curdir)
